/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>

#include "cpucaanoo.h"

 /*
 * quick tool to set LCD timings for Wiz
 * (c) notaz, 2009
 * code dedicated to public domain.
 *
 * HTOTAL:    X VTOTAL:  341
 * HSWIDTH:   1 VSWIDTH:   0
 * HASTART:  37 VASTART:  17
 * HAEND:   277 VAEND:   337
 *
 * 120Hz
 * pcd  8, 447: + 594us
 * pcd  9, 397: +  36us
 * pcd 10, 357: - 523us
 * pcd 11, 325: +1153us
 *
 * 'lcd_timings=397,1,37,277,341,0,17,337;clkdiv0=9'
 *
 * set LCD timings based on preformated string (see usage() below for params).
 * returns 0 on success.
 */

/* Declaracion de variables globales */
#define SYS_CLK_FREQ 27

static          unsigned int    gp2x_dev = 0;
static volatile unsigned short *gp2x_memregs;
static volatile unsigned long  *gp2x_memregl;

static void 
loc_set_FCLK(unsigned int clock_in_mhz)
{
  unsigned int v;
  unsigned int mdiv, pdiv, sdiv = 0;
  pdiv = 9;
  mdiv = (clock_in_mhz * pdiv) / SYS_CLK_FREQ;
  mdiv &= 0x3ff;
  v = (pdiv<<18) | (mdiv<<8) | sdiv;

  gp2x_memregl[0xf004>>2] = v;
  gp2x_memregl[0xf07c>>2] |= 0x8000;

  while (gp2x_memregl[0xf07c>>2] & 0x8000) ;
}

static unsigned int loc_clock_in_mhz = GP2X_DEF_CLOCK;

static void 
loc_cpu_set_clock(unsigned int clock_in_mhz)
{
  if (clock_in_mhz == loc_clock_in_mhz) return;
  loc_clock_in_mhz = clock_in_mhz;
  
  loc_set_FCLK(clock_in_mhz);
  return;
}


void 
cpu_init()
{
  gp2x_dev = open("/dev/mem", O_RDWR);
  gp2x_memregl = (unsigned long  *)mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev, 0xc0000000);
  gp2x_memregs = (unsigned short *)gp2x_memregl;
  loc_cpu_set_clock( GP2X_DEF_EMU_CLOCK );
}

void 
cpu_deinit()
{
  loc_cpu_set_clock( GP2X_DEF_CLOCK );
  sleep(1);
  munmap((void *)gp2x_memregl, 0x10000);
  close(gp2x_dev);
  fcloseall();
  sync();
}

void
cpu_set_clock(unsigned int clock_in_mhz)
{
  // nothing to do
}

unsigned int
cpu_get_clock()
{
  return loc_clock_in_mhz;
}

